<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Chat;
use App\Models\Notification;
use App\Models\Room;
use App\Models\UserToken;
use Carbon\Carbon;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use JWTAuth;

class ChatController extends Controller
{
    public function chat_history()
    {

        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $rooms = Room::where('sender_id', $user->id)->orWhere('receiver_id', $user->id)->orderByDesc('updated_at')->whereHas('chats')->get();
        $data = [];
        foreach ($rooms as $room) {

            $arr['id'] = $room->id;
            $arr['time'] = Carbon::parse($room->updated_at)->diffForHumans();
            $last_message = $room->chats()->OrderByDesc('id')->first();
            $arr['sender_name'] = $last_message->sender->username;
            $arr['last_message'] = $last_message->message;
            $arr['image'] = $last_message->sender->image ? asset('images/users/' . $last_message->sender->image) : '';
            $arr['chats'] = [];
            foreach ($room->chats()->OrderByDesc('id')->get() as $message) {
                $ch['id'] = $message->id;
                $ch['sender_id'] = $message->sender_id;
                $ch['sender_name'] = $message->sender->username;
                $ch['sender_image'] = $message->sender->image ? asset('images/users/' . $message->sender->image) : '';
                $ch['receiver_id'] = $message->receiver_id;
                $ch['receiver_name'] = $message->receiver->username;
                $ch['receiver_image'] = $message->receiver->image ? asset('images/users/' . $message->receiver->image) : '';
                $ch['message'] = $message->message;
                $ch['time'] = Carbon::parse($message->created_at)->diffForHumans();
                $arr['chats'][] = $ch;
            }
            $data[] = $arr;
        }
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function check_room($receiver_id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $room = Room::where(['sender_id' => $user->id, 'receiver_id' => $receiver_id])->orWhere(['sender_id' => $receiver_id, 'receiver_id' => $user->id])->whereHas('chats')->latest()->first();
        if (isset($room)) {

            $data['id'] = $room->id;
            $data['time'] = Carbon::parse($room->updated_at)->diffForHumans();
            $last_message = $room->chats()->OrderByDesc('id')->first();
            $data['sender_name'] = $last_message->sender->username;
            $data['last_message'] = $last_message->message;
            $data['image'] = $last_message->sender->image ? asset('images/users/' . $last_message->sender->image) : '';
            $data['chats'] = [];
            foreach ($room->chats()->OrderByDesc('id')->get() as $message) {
                $ch['id'] = $message->id;
                $ch['sender_id'] = $message->sender_id;
                $ch['sender_name'] = $message->sender->username;
                $ch['sender_image'] = $message->sender->image ? asset('images/users/' . $message->sender->image) : '';
                $ch['receiver_id'] = $message->receiver_id;
                $ch['receiver_name'] = $message->receiver->username;
                $ch['receiver_image'] = $message->receiver->image ? asset('images/users/' . $message->receiver->image) : '';
                $ch['message'] = $message->message;
                $ch['time'] = Carbon::parse($message->created_at)->diffForHumans();
                $data['chats'][] = $ch;
            }


        } else {
            $new_room = new Room();
            $new_room->sender_id = $user->id;
            $new_room->receiver_id = $receiver_id;
            $new_room->save();
            $data['id'] = $new_room->id;
            $data['time'] = Carbon::parse($new_room->created_at)->diffForHumans();
            $data['sender_name'] = '';
            $data['last_message'] = '';
            $data['image'] = '';
            $data['chats'] = [];


        }
        return response()->json(['value' => true, 'data' => $data]);
    }


    public function send_message(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'receiver_id' => 'required|exists:users,id',
            'room_id' => 'required|exists:rooms,id',
            'message' => 'required'
        ]);
        if ($validator->passes()) {
            $room = Room::find($request['room_id']);
            $room->updated_at = Carbon::now();
            $room->update();
            $message = new Chat();
            $message['sender_id'] = $user->id;
            $message['receiver_id'] = $request['receiver_id'];
            $message['message'] = $request['message'];
            $message['room_id'] = $request['room_id'];
            $message->save();

            $notification = new Notification();
            $notification['user_id'] = $request['receiver_id'];
            $notification['body->ar'] = $request['message'];
            $notification['body->en'] = $request['message'];;
            $notification['type'] = 'message';
            $notification->save();
            $tokens = UserToken::where('user_id', $request['receiver_id'])->get();
            $push = new PushNotification('fcm');
            foreach ($tokens as $token) {


                if ($token->lang == 'ar') {
                    $title = $notification->body['ar'];
                    $body = $notification->body['ar'];
                } else {

                    $title = $notification->body['en'];
                    $body = $notification->body['en'];

                }

                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification['body->ar'],
                            'title_en' => $notification['body->en'],
                            'body_ar' => $notification['body->ar'],
                            'body_en' => $notification['body->en'],
                            'type' => 'message',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => $notification['body->ar'],
                            'title_en' => $notification['body->en'],
                            'body_ar' => $notification['body->ar'],
                            'body_en' => $notification['body->en'],
                            'type' => 'message',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification['body->ar'],
                            'title_en' => $notification['body->en'],
                            'body_ar' => $notification['body->ar'],
                            'body_en' => $notification['body->en'],
                            'type' => 'message',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                }

            }






            $data['id'] = $room->id;
            $data['time'] = Carbon::parse($room->updated_at)->diffForHumans();
            $last_message = $room->chats()->OrderByDesc('id')->first();
            $data['sender_name'] = $last_message->sender->username;
            $data['last_message'] = $last_message->message;
            $data['image'] = $last_message->sender->image ? asset('images/users/' . $last_message->sender->image) : '';
            $data['chats'] = [];
            foreach ($room->chats()->latest()->get() as $message) {
                $ch['id'] = $message->id;
                $ch['sender_id'] = $message->sender_id;
                $ch['sender_name'] = $message->sender->username;
                $ch['sender_image'] = $message->sender->image ? asset('images/users/' . $message->sender->image) : '';
                $ch['receiver_id'] = $message->receiver_id;
                $ch['receiver_name'] = $message->receiver->username;
                $ch['receiver_image'] = $message->receiver->image ? asset('images/users/' . $message->receiver->image) : '';
                $ch['message'] = $message->message;
                $ch['time'] = Carbon::parse($message->created_at)->diffForHumans();
                $data['chats'][] = $ch;
            }

            return response()->json(['value' => true, 'data' => $data]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }
}
